/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: lpq.c,v 1.2 2002/10/24 06:12:57 jacobs Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libintl.h>
#include <papi.h>

static int type = 3; /* lpq short report format */
static int debug = 0;

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, "Usage: %s [-P printer] (user|id ...)\n", name);
	exit (1);
}

static char *
state_string(int state)
{
	switch (state) {
	case 3:
		return ("idle");
	case 4:
		return ("processing");
	case 5:
		return ("stopped");
	default:
		return ("unknown");
	}
}

static char *_rank_suffixes[] = {
	"th", "st", "nd", "rd", "th", "th", "th", "th", "th", "th"
};


static char *
rank_string(const int rank)
{
 	static char buf[12];

	if (rank < 0)
		snprintf(buf, sizeof (buf), gettext("invalid"));
	else if (rank == 0)
		snprintf(buf, sizeof (buf), gettext("active"));
	else if ((rank > 10) && (rank < 14))
		sprintf(buf, "%dth", rank);
	else
            	sprintf(buf, "%d%s", rank, _rank_suffixes[rank % 10]);

	return (buf);
}

static void
printer_state_line(char *name)
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_printer_t p = NULL;
	const char *pattrs[] = {"printer-state", "printer-state-reasons", NULL};

	status = papiServiceCreate(&svc, name, NULL, NULL, NULL,
					PAPI_ENCRYPT_NEVER, NULL);
	if (status != PAPI_OK) {
		char *mesg;

		if (((mesg = papiServiceGetStatusMessage(svc)) == NULL) &&
		    ((mesg = papiStatusString(status)) == NULL))
			mesg = "unknown";

		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), name, mesg);
		exit(1);
	}

	if ((status = papiPrinterQuery(svc, name, pattrs, NULL, &p)) == PAPI_OK) {
		papi_attribute_t **list = papiPrinterGetAttributeList(p);
		int state = 0;
		char *reason = "";

		papiAttributeListGetInteger(list, NULL,
					"printer-state", &state);
		papiAttributeListGetString(list, NULL,
					"printer-state-reasons", &reason);

		printf("%s: %s", name, state_string(state));
		if (state == 0x05) /* stopped */
			printf("%s\n", reason);
		printf("\n");
		if (debug != 0)
			papiAttributeListPrint(stderr, "printer: ", list);
	} else {
		char *mesg;

		if (((mesg = papiServiceGetStatusMessage(svc)) == NULL) &&
		    ((mesg = papiStatusString(status)) == NULL))
			mesg = "unknown";

		fprintf(stderr, gettext("%s: %s\n"), name, mesg);
		exit(1);
	}

	papiPrinterFree(p);
	papiServiceDestroy(svc);
}

static void
print_job_line(papi_job_t job)
{
	papi_attribute_t **list = papiJobGetAttributeList(job);
	int id = 0, rank = 0, size = 0;
	char *name = "print job";
	char *user = "unknown";
	char *host = "unknown";
	char *suffix = "k";

	papiAttributeListGetInteger(list, NULL,
					"job-id", &id);
	papiAttributeListGetInteger(list, NULL,
					"number-of-intervening-jobs", &rank);

	if (papiAttributeListGetInteger(list, NULL, "job-octets", &size)
			== PAPI_OK)
		suffix = "bytes";
	else
		papiAttributeListGetInteger(list, NULL,
					"job-k-octets", &size);
	papiAttributeListGetString(list, NULL,
					"job-name", &name);
	papiAttributeListGetString(list, NULL,
					"job-originating-user-name", &user);

	if (type == 3)
		printf(gettext("%s\t%s\t%d\t%-32.32s%d %s\n"),
			rank_string(++rank), user, id, name, size, suffix);
	else
		printf(gettext("\n%s: %s\t\t\t\t[job %d %s]\n\t%-32.32s\t%d %s\n"),
			user, rank_string(++rank), id, host, name, size, suffix);

	if (debug != 0)
		papiAttributeListPrint(stderr, "job: ", list);
}

static void
job_lines(char *name)
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_job_t *jobs = NULL;
	const char *jattrs[] = {
			"job-name", "job-octets", "job-k-octets",
			"job-originating-user-name", "job-id",
			"number-of-intervening-jobs", NULL};

	status = papiServiceCreate(&svc, name, NULL, NULL, NULL,
					PAPI_ENCRYPT_NEVER, NULL);
	if (status != PAPI_OK) {
		char *mesg;

		if (((mesg = papiServiceGetStatusMessage(svc)) == NULL) &&
		    ((mesg = papiStatusString(status)) == NULL))
			mesg = "unknown";

		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), name, mesg);
		exit(1);
	}

	status = papiPrinterListJobs(svc, name, jattrs, 0, 0, &jobs);
	if (status == PAPI_OK) {
		if (jobs != NULL) {
			int i;

			if (type == 3)
#define SHORT_HEADER	"Rank\tOwner\tJob\tFile(s)\t\t\t\tTotal Size\n"
				printf(gettext(SHORT_HEADER));
#undef SHORT_HEADER

			for (i = 0; jobs[i] != NULL; i++)
				print_job_line(jobs[i]);
		}
	} else {
		char *mesg;

		if (((mesg = papiServiceGetStatusMessage(svc)) == NULL) &&
		    ((mesg = papiStatusString(status)) == NULL))
			mesg = "unknown";

		fprintf(stderr, gettext("%s: %s\n"), name, mesg);
		exit(1);
	}

	papiJobListFree(jobs);
	papiServiceDestroy(svc);
}



int
main(int ac, char *av[])
{
	int exit_code = 0;
	char *printer = NULL;
	char *password = NULL;
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_printer_t p = NULL;
	int c;
	const char *pattrs[] = {"printer-state", "printer-state-reason", NULL};

	while ((c = getopt(ac, av, "DP:l")) != EOF)
		switch (c) {
		case 'D':
			debug++;
			break;
		case 'P':
			printer = optarg;
			break;
		case 'p':
			password = optarg;
			break;
		case 'l':
			type = 4;
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	printer_state_line(printer);
	job_lines(printer);

	for (c = optind; c < ac; c++) {
		char *user = NULL;
		char *value = av[c];
		int id = -1;

		if (isdigit(value[0]) != 0)
			id = atoi(value);
		else
			user = value; 

		status = papiServiceCreate(&svc, value, user, password, NULL,
					encryption, NULL);
		if (status != PAPI_OK) {
			char *mesg = papiServiceGetStatusMessage(svc);

			if (mesg == NULL)
				mesg = papiStatusString(status);

			fprintf(stderr, gettext(
				"Failed to contact service for %s: %s\n"),
				value, mesg);
			exit(1);
		}

		if (id != -1) { /* is an individual job */
			papi_job_t job = NULL;
			char *mesg;

			status = papiJobQuery(svc, printer, id, NULL, &job);
			if (status != PAPI_OK) {
				mesg = papiServiceGetStatusMessage(svc);
				if (mesg == NULL)
					mesg = papiStatusString(status);
				fprintf(stderr, "%d: %s\n", id, mesg);
				exit_code = 1;
			} else
				mesg = "cancelled";
			fprintf(stderr, "%s-%d: %s\n", printer, id, mesg);
		}
	
		papiServiceDestroy(svc);
	}

	exit(exit_code);
}
