/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: service.c,v 1.2 2002/09/20 05:51:03 jacobs Exp $ */
#pragma ident	"@(#)service.c	1.1	02/08/22 SMI"

/*LINTLIBRARY*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <papi_impl.h>

papi_status_t
papiServiceCreate(papi_service_t *handle, const char *service_name,
		const char *user_name, const char *password,
		const int (*authCB)(papi_service_t svc),
		const papi_encryption_t encryption, void *app_data)
{
	service_t *svc = NULL;
	
	if (handle == NULL)
		return (PAPI_BAD_ARGUMENT);

	if ((*handle = svc = calloc(1, sizeof (*svc))) == NULL)
		return (PAPI_TEMPORARY_ERROR);

	if (service_name != NULL)
		papiAttributeListAddString(&svc->attributes, PAPI_ATTR_EXCL,
				"service-name", service_name);

	(void) papiServiceSetUsername(svc, user_name);
	(void) papiServiceSetPassword(svc, password);
	(void) papiServiceSetAuthCB(svc, authCB);
	(void) papiServiceSetAppData(svc, app_data);
	(void) papiServiceSetEncryption(svc, encryption);

	return (PAPI_OK);
}

void
papiServiceDestroy(papi_service_t handle)
{
	service_t *svc = handle;

	if (svc != NULL) {
		papiAttributeListFree(svc->attributes);
		free(handle);
	}
}

papi_status_t
papiServiceSetUsername(papi_service_t handle, const char *user_name)
{
	service_t *svc = handle;

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	return (papiAttributeListAddString(&svc->attributes, PAPI_ATTR_REPLACE,
				"user-name", user_name));
}

papi_status_t
papiServiceSetPassword(papi_service_t handle, const char *password)
{
	service_t *svc = handle;

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	return (papiAttributeListAddString(&svc->attributes, PAPI_ATTR_REPLACE,
				"password", password));
}

papi_status_t
papiServiceSetEncryption(papi_service_t handle,
			const papi_encryption_t encryption)
{
	service_t *svc = handle;

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	return (papiAttributeListAddInteger(&svc->attributes, PAPI_ATTR_REPLACE,
				"encryption", (int)encryption));
}

papi_status_t
papiServiceSetAuthCB(papi_service_t handle,
			const int (*authCB)(papi_service_t svc))
{
	service_t *svc = handle;

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	svc->authCB = (int (*)(papi_service_t svc))authCB;

	return (PAPI_OK);
}

papi_status_t
papiServiceSetAppData(papi_service_t handle, const void *app_data)
{
	service_t *svc = handle;

	if (svc == NULL)
		return (PAPI_BAD_ARGUMENT);

	svc->app_data = (void *)app_data;

	return (PAPI_OK);
}

char *
papiServiceGetServicename(papi_service_t handle)
{
	service_t *svc = handle;
	char *result = NULL;

	if (svc != NULL)
		papiAttributeListGetString(svc->attributes, NULL,
					"service-name", &result);

	return (result);
}

char *
papiServiceGetUsername(papi_service_t handle)
{
	service_t *svc = handle;
	char *result = NULL;

	if (svc != NULL)
		papiAttributeListGetString(svc->attributes, NULL,
					"user-name", &result);

	return (result);
}

char *
papiServiceGetPassword(papi_service_t handle)
{
	service_t *svc = handle;
	char *result = NULL;

	if (svc != NULL)
		papiAttributeListGetString(svc->attributes, NULL,
					"password", &result);

	return (result);
}

papi_encryption_t
papiServiceGetEncryption(papi_service_t handle)
{
	service_t *svc = handle;
	papi_encryption_t result = PAPI_ENCRYPT_NEVER;

	if (svc != NULL)
		papiAttributeListGetInteger(svc->attributes, NULL,
					"encryption", (int *)&result);

	return (result);
}

void *
papiServiceGetAppData(papi_service_t handle)
{
	service_t *svc = handle;
	void *result = NULL;

	if (svc != NULL)
		result = svc->app_data;

	return (result);
}

char *
papiServiceGetStatusMessage(papi_service_t handle)
{
	service_t *svc = handle;
	char *result = NULL;

	if (svc != NULL) {
		papiAttributeListGetString(svc->attributes, NULL,
					"status-message", &result);
		papiAttributeListGetString(svc->attributes, NULL,
					"detailed-status-message", &result);
	}

	return (result);
}

void
detailed_error(service_t *svc, char *fmt, ...)
{
	va_list ap;
	char buf[BUFSIZ];

	if ((svc == NULL) || (fmt == NULL))
		return;

	va_start(ap, fmt);
#ifdef HAVE_SNPRINTF
	vsnprintf(buf, sizeof (buf), fmt, ap);
#else
	vsprintf(buf, fmt, ap);
#endif
	va_end(ap);

	papiAttributeListAddString(&svc->attributes, PAPI_ATTR_APPEND,
				"detailed-status-message", buf);
}
