/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: lp.c,v 1.2 2002/10/24 06:13:29 jacobs Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libintl.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout,
		gettext("Usage: %s [-c] [-m] [-p] [-s] [-w] [-d destination]  "
			"[-f form-name] [-H special-handling] [-n number] "
			"[-o option] [-P page-list] [-q priority-level]  "
			"[-S character-set | print-wheel]  [-t title] "
			"[-T content-type [-r]] [-y mode-list] [file...]\n"),
		name);
	exit (1);
}

static struct {
	char *mime_type;
	char *lp_type;
} type_map[] = {
	{ "plain/text", "simple" },
	{ "application/octet-stream", "raw" },
	{ "application/octet-stream", "any" },
	{ "application/postscript", "postscript" },
	{ "application/postscript", "ps" },
	{ "application/x-cif", "cif" },
	{ "application/x-dvi", "dvi" },
	{ "application/x-plot", "plot" },
	{ "application/x-ditroff", "troff" },
	{ "application/x-troff", "otroff" },
	{ "application/x-pr", "pr" },
	{ "application/x-fortran", "fortran" },
	{ "application/x-raster", "raster" },
	{ NULL, NULL}
};

static char *
lp_type_to_mime_type(char *lp_type)
{
 	int i;

	if (lp_type == NULL)
		return ("application/octet-stream");

	for (i = 0; type_map[i].lp_type != NULL; i++)
		if (strcasecmp(type_map[i].lp_type, lp_type) == 0)
			return (type_map[i].mime_type);

	return (lp_type);
}

/* this should be in a shared library for lp(1) and lpr(1b) to share it */
static papi_status_t
jobSubmitSTDIN(papi_service_t svc, char *printer, papi_attribute_t **list,
		papi_job_t *job)
{
	papi_status_t status;
	papi_stream_t stream = NULL;
	int rc;
	char buf[BUFSIZ];

	status = papiJobStreamOpen(svc, printer,
				(const papi_attribute_t **)list, NULL, &stream);

	while ((status == PAPI_OK) && ((rc = read(0, buf, sizeof (buf))) > 0))
		status = papiJobStreamWrite(svc, stream, buf, rc);

	if (status == PAPI_OK)
		status = papiJobStreamClose(svc, stream, job);

	return (status);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_attribute_t **list = NULL;
	papi_job_t job = NULL;
	int exit_code = 0;
	char *printer = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int copy = 0;
	int silent = 0;
	int c;
	int modify = -1;

	while ((c = getopt(ac, av, "H:P:S:T:cd:f:i:mn:o:pq:rst:wy:")) != EOF)
		switch (c) {
		case 'H':	/* handling */
			if (strcasecmp(optarg, "hold") == 0)
				papiAttributeListAddString(&list,
					PAPI_ATTR_EXCL,
					"job-hold-until", "indefinite");
			else if (strcasecmp(optarg, "release") == 0)
				papiAttributeListAddString(&list,
					PAPI_ATTR_EXCL,
					"job-hold-until", "no-hold");
			else if (strcasecmp(optarg, "immediate") == 0)
				papiAttributeListAddInteger(&list,
					PAPI_ATTR_EXCL,
					"job-priority", 100);
			else
				papiAttributeListAddString(&list,
					PAPI_ATTR_EXCL,
					"job-hold-until", optarg);
			break;
		case 'P':	/* page list */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"page-ranges", optarg);
			break;
		case 'S':	/* charset */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"lp-charset", optarg);
			break;
		case 'T':	/* type */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"document-format",
					lp_type_to_mime_type(optarg));
			break;
		case 'c':	/* copy */
			copy = 1;
			break;
		case 'd':	/* destination */
			printer = optarg;
			break;
		case 'f':	/* form */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"media", optarg);
			break;
		case 'i':	/* modify job */
			modify = atoi(optarg);
			break;
		case 'm':	/* mail when complete */
			papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
				"rfc-1179-mail", 1);
			break;
		case 'n':	/* copies */
			papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"copies", atoi(optarg));
			break;
		case 'o':	/* lp "options" */
			if (strstr(optarg, "nobanner") == 0)
				papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-sheets", "none");
			papiAttributeListAddString(&list, PAPI_ATTR_APPEND,
					"lp-options", optarg);
			break;
		case 'p':	/* Solaris - notification */
			papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
				"rfc-1179-mail", 1);
			break;
		case 'q': {	/* priority */
			int i = atoi(optarg);

			i = 99 * (39 - i) / 39 + 1;
			if ((i < 1) || (i > 100)) {
				fprintf(stderr, gettext(
					"priority must be between 0 and 39.\n")
					);
				exit (1);
			}	
			papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"priority", i);
			}
			break;
		case 'r':	/* "raw" mode */
			papiAttributeListAddString(&list, PAPI_ATTR_APPEND,
				"lp-options", "stty=raw");
			break;
		case 's':	/* suppress message */
			silent = 1;
			break;
		case 't':	/* title */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-name", optarg);
			break;
		case 'w':
			papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
				"rfc-1179-mail", 1);
			break;
		case 'y':	/* lp "modes" */
			papiAttributeListAddString(&list, PAPI_ATTR_APPEND,
					"lp-modes", optarg);
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL, "copies", 1);
	papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/octet-stream");
	papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"job-sheets", "standard");
			

	status = papiServiceCreate(&svc, printer, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		char *mesg = papiServiceGetStatusMessage(svc);

		if (mesg == NULL)
			mesg = papiStatusString(status);

		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			mesg);
		exit(1);
	}

	if (modify != -1)
		status = papiJobModify(svc, printer, modify,
					(const papi_attribute_t **)list, &job);
	if (optind == ac)	/* no file list, use stdin */
		status = jobSubmitSTDIN(svc, printer, list, &job);
	else if (copy == 0)	/* reference the files in the job, default */
		status = papiJobSubmitByReference(svc, printer,
					(const papi_attribute_t **)list,
					NULL, (const char **)&av[optind], &job);
	else			/* copy the files before return, -c */
		status = papiJobSubmit(svc, printer,
					(const papi_attribute_t **)list,
					NULL, (const char **)&av[optind], &job);

	if (status != PAPI_OK) {
		char *mesg = papiServiceGetStatusMessage(svc);
		if (mesg == NULL) {
			mesg = papiStatusString(status);
			exit_code = 1;
		} else
			mesg = "failed";
		fprintf(stderr, "%s: %s\n", printer, mesg);
	}

	papiAttributeListFree(list);

	if ((silent == 0) &&
	    ((list = papiJobGetAttributeList(job)) != NULL)) {
		char *tmp = "unknown";

		papiAttributeListGetString(list, NULL, "lpsched-id", &tmp);
		printf("request id is %s (%d file(s))\n", tmp, ac - optind);
	}	

	papiJobFree(job);
	papiServiceDestroy(svc);

	exit(exit_code);
}
